package net.sf.corn.gate;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;


/**
 * This class defines the meta data of a service operation. 
 * 
 * @author Serhat Dirik
 */
public class OperationDefinition implements Cloneable , Serializable  {
 
	private static final long serialVersionUID = 744960800046280077L;
	
	private String name=null;
	private String methodTarget=null;
	private String description=null;
	private boolean authenticationRequiredToInspect = false;
	private boolean authenticationRequiredToUse     = false;
	private CopyOnWriteArrayList<String> authorizedRolesToInspect = new CopyOnWriteArrayList<String>();
	private CopyOnWriteArrayList<String> authorizedRolesToUse = new CopyOnWriteArrayList<String>();
	private boolean secureProtocolRequiredToInspect = false;
	private boolean secureProtocolRequiredToUse     = false;
	private boolean remoteAccessEnabled     = true;
	/**
	 * Constructor to clone a new definition from the existent one 
	 * @param op an operation definition to be cloned into
	 */
	public OperationDefinition(OperationDefinition op){
		if(op == null ) {
			throw new IllegalArgumentException("Operation definition parameter is null!");
		}
		this.authenticationRequiredToInspect = op.authenticationRequiredToInspect;
		this.authenticationRequiredToUse = op.authenticationRequiredToUse;
		this.description = op.description;
		this.name = op.name;
		this.methodTarget =op.methodTarget;
		this.secureProtocolRequiredToInspect = op.secureProtocolRequiredToInspect;
		this.secureProtocolRequiredToUse = op.secureProtocolRequiredToUse;
		this.remoteAccessEnabled = op.remoteAccessEnabled;
		for (String role : op.getAuthorizedRolesToInspect()) {
			this.addAuthorizedRoleToInspect(role); 
		}
		for (String role : op.getAuthorizedRolesToUse()) {
			this.addAuthorizedRoleToUse(role); 
		}
	}
	
	OperationDefinition(Method method){
		if( method.isAnnotationPresent(GateOperation.class) ){
			GateOperation op = method.getAnnotation(GateOperation.class);
			
			this.authenticationRequiredToInspect = op.authenticationRequiredToInspect();
			this.authenticationRequiredToUse = op.authenticationRequiredToUse();
			this.description = op.description()!= null && op.description().trim().length() > 0 ? op.description() : null;
			this.name = op.name() != null && op.name().trim().length() > 0 ? op.name() : method.getName();
			this.methodTarget =method.getName();
			this.secureProtocolRequiredToInspect = op.secureProtocolRequiredToInspect();
			this.secureProtocolRequiredToUse = op.secureProtocolRequiredToUse();
			this.remoteAccessEnabled = op.remoteAccessEnabled();
			for (String role : op.authorizedRolesToInspect()) {
				this.addAuthorizedRoleToInspect(role); 
			}
			for (String role : op.authorizedRolesToUse()) {
				this.addAuthorizedRoleToUse(role); 
			}
		}else{
			this.name = method.getName();
			this.methodTarget =method.getName();
		}
	}
	/**
	 * Defines a new op and gives it a name
	 * @param name name of the operation
	 */
	public OperationDefinition(String name) {
		if(name == null || name.trim().length() == 0) {
			throw new IllegalArgumentException("Name parameter is null or empty");
		}
		
		this.name = name;
	} 
	/**
	 * Defines a new op, gives it a name and sets its target (mapped) method
	 * @param name name of the operation
	 * @param target name of the mapped method
	 */
	public OperationDefinition(String name, String target) {
		if(name == null || name.trim().length() == 0) {
			throw new IllegalArgumentException("Name parameter is null or empty");
		}
		
		this.name = name;
		this.methodTarget = target;
	}
	/**
	 * Another contructor that also sets the description
	 * @param name name of the op 
	 * @param target the mapped method name 
	 * @param description a description about the operation
	 */
	public OperationDefinition(String name, String target,String description) {
		if(name == null || name.trim().length() == 0) {
			throw new IllegalArgumentException("Name parameter is null or empty");
		}
		
		this.name = name;
		this.methodTarget = target;
		this.description = description;
	}
	
	/**
	 * Gives the name of the operation
	 * @return name of the operation
	 */
	public String getName(){
		return name;
	}
	
	protected void setName(String name){
		this.name = name;
	}
	/**
	 * Gives the target method name
	 * @return the target method name
	 */
	public String getMethodTarget() {
		return methodTarget == null ? getName() : methodTarget;
	}

	
	protected void setMethodTarget(String methodTarget) {
		this.methodTarget = methodTarget;
	}
	/**
	 * Tells if a secure protocol like HTTPS required to inspect this operation
	 * @return true if secure protocol required to inspect
	 */
	public boolean isSecureProtocolRequiredToInspect() {
		return secureProtocolRequiredToInspect;
	}
	/**
	 * Sets the secure protocol requirement to inspect 
	 * @param secureProtocolRequiredToInspect true if secure protocol required
	 */
	public void setSecureProtocolRequiredToInspect(
			boolean secureProtocolRequiredToInspect) {
		this.secureProtocolRequiredToInspect = secureProtocolRequiredToInspect;
	}
	/**
	 * Tells if a secure protocol like HTTPS required to use this operation
	 * @return true if a secure protocol required to use
	 */
	public boolean isSecureProtocolRequiredToUse() {
		return secureProtocolRequiredToUse;
	}
	/**
	 * Sets the secure protocol requirement to use
	 * @param secureProtocolRequiredToUse true if a secure protocol required to use this op
	 */
	public void setSecureProtocolRequiredToUse(boolean secureProtocolRequiredToUse) {
		this.secureProtocolRequiredToUse = secureProtocolRequiredToUse;
	}
	/**
	 * Tells whether this operation is enabled to access from remote
	 * @return true if this operation can be accessible from the remote
	 */
	public boolean isRemoteAccessEnabled() {
		return remoteAccessEnabled;
	}
	/**
	 * Enable/disable remote access to the operation
	 * @param remoteAccessEnabled true if the remote access is enabled
	 */
	public void setRemoteAccessEnabled(boolean remoteAccessEnabled) {
		this.remoteAccessEnabled = remoteAccessEnabled;
	}
	/**
	 * Sets description 
	 * @param description a description text
	 */
	public void setDescription(String description) {
		this.description = description;
	}
	/**
	 * Gives description about the operation
	 * @return a description string
	 */
	public String getDescription() {
		return description;
	}
	/**
	 * Tells if authentication required to inspect this service operation 
	 * @return true if authentication required to inspect
	 */
	public boolean isAuthenticationRequiredToInspect() {
		return authorizedRolesToInspect.size() > 0 ||  authenticationRequiredToInspect;
	}

	/**
	 * Sets authentication requirement to inspect this operation
	 * @param authenticationRequiredToInspect true if authentication required 
	 */
	public void setAuthenticationRequiredToInspect(
			boolean authenticationRequiredToInspect) {
		this.authenticationRequiredToInspect = authenticationRequiredToInspect;
	}
	/**
	 * Sets authentication requirement to use this operation
	 * @param authenticationRequiredToUse true if authentication required
	 */
	public void setAuthenticationRequiredToUse(
			boolean authenticationRequiredToUse) {
		this.authenticationRequiredToUse = authenticationRequiredToUse;
	}
	/**
	 * Tells if authentication required to use
	 * @return true if authentication required
	 */
	public boolean isAuthenticationRequiredToUse() {
		return authorizedRolesToUse.size() > 0 || authenticationRequiredToUse;
	}

	/**
	 * Gives list of authorized roles to inspect this operation
	 * @return list of authorized role names
	 */
	public List<String> getAuthorizedRolesToInspect() {
		return Collections.unmodifiableList(authorizedRolesToInspect);
	}
	/**
	 * Adds a role to authorized user list to inspect this operation
	 * @param role role name
	 * @return true if the given role is successfully added
	 */
	public boolean addAuthorizedRoleToInspect(String role){
		if(role == null || role.trim().length() == 0){
			throw new IllegalArgumentException("Role name parameter is null or empty");
		}
		if( authorizedRolesToInspect.contains(role.trim())) {
			return true;
		}
		return authorizedRolesToInspect.add(role.trim());
	}
	/**
	 * Removes the given role from the authorized users list to inspect 
	 * @param role role name
	 * @return true if successfully removed
	 */
	public boolean removeAuthorizedRoleToInspect(String role){
		if(role == null || role.trim().length() == 0){
			throw new IllegalArgumentException("Role name parameter is null or empty");
		}
		if( authorizedRolesToInspect.contains(role.trim())) {
			return authorizedRolesToInspect.remove(role.trim());
		}
		return true;
	}
	
	/**
	 * Gives the list of authorized users to use this operation
	 * @return list of authorized users
	 */
	public List<String> getAuthorizedRolesToUse() {
		return Collections.unmodifiableList(authorizedRolesToUse);
	}
	/**
	 * Adds an  user to authorized users list to use
	 * @param role role name
	 * @return true if successfully added
	 */
	public boolean addAuthorizedRoleToUse(String role){
		if(role == null || role.trim().length() == 0){
			throw new IllegalArgumentException("Role name parameter is null or empty");
		}
		if( authorizedRolesToUse.contains(role.trim())) {
			return true;
		}
		return authorizedRolesToUse.add(role.trim());
	}
	/**
	 * Removes the given role from the authorized users list to use
	 * @param role role name 
	 * @return true if successfully removed
	 */
	public boolean removeAuthorizedRoleToUse(String role){
		if(role == null || role.trim().length() == 0){
			throw new IllegalArgumentException("Role name parameter is null or empty");
		}
		if( authorizedRolesToUse.contains(role.trim())) {
			return authorizedRolesToUse.remove(role.trim());
		}
		return true;
	}
	

	@Override
	public OperationDefinition clone() throws CloneNotSupportedException {
		OperationDefinition clone = new OperationDefinition(name, methodTarget,description);
		clone.setAuthenticationRequiredToInspect(authenticationRequiredToInspect);
		clone.setAuthenticationRequiredToUse(authenticationRequiredToUse);
		for (String role : authorizedRolesToInspect) {
			clone.addAuthorizedRoleToInspect(role); 
		}
		for (String role : authorizedRolesToUse) {
			clone.addAuthorizedRoleToUse(role); 
		}
		return clone;
	}

	@Override
	public String toString() {
		return new StringBuilder().append("<Operation> ").append("Name:").append(name)
				.append(", Target:").append(methodTarget).toString();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ (authenticationRequiredToInspect ? 1231 : 1237);
		result = prime * result + (authenticationRequiredToUse ? 1231 : 1237);
		result = prime
				* result
				+ ((authorizedRolesToInspect == null) ? 0
						: authorizedRolesToInspect.hashCode());
		result = prime
				* result
				+ ((authorizedRolesToUse == null) ? 0 : authorizedRolesToUse
						.hashCode());
		result = prime * result
				+ ((description == null) ? 0 : description.hashCode());
		result = prime * result
				+ ((methodTarget == null) ? 0 : methodTarget.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + (remoteAccessEnabled ? 1231 : 1237);
		result = prime * result
				+ (secureProtocolRequiredToInspect ? 1231 : 1237);
		result = prime * result + (secureProtocolRequiredToUse ? 1231 : 1237);
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		OperationDefinition other = (OperationDefinition) obj;
		if (authenticationRequiredToInspect != other.authenticationRequiredToInspect)
			return false;
		if (authenticationRequiredToUse != other.authenticationRequiredToUse)
			return false;
		if (authorizedRolesToInspect == null) {
			if (other.authorizedRolesToInspect != null)
				return false;
		} else if (!authorizedRolesToInspect
				.equals(other.authorizedRolesToInspect))
			return false;
		if (authorizedRolesToUse == null) {
			if (other.authorizedRolesToUse != null)
				return false;
		} else if (!authorizedRolesToUse.equals(other.authorizedRolesToUse))
			return false;
		if (description == null) {
			if (other.description != null)
				return false;
		} else if (!description.equals(other.description))
			return false;
		if (methodTarget == null) {
			if (other.methodTarget != null)
				return false;
		} else if (!methodTarget.equals(other.methodTarget))
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (remoteAccessEnabled != other.remoteAccessEnabled)
			return false;
		if (secureProtocolRequiredToInspect != other.secureProtocolRequiredToInspect)
			return false;
		if (secureProtocolRequiredToUse != other.secureProtocolRequiredToUse)
			return false;
		return true;
	}
	
}
