package net.sf.corn.gate.jsonrpc;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import net.sf.corn.converter.ConversionException;
import net.sf.corn.converter.ParsingException;
import net.sf.corn.converter.json.JsTypeComplex;
import net.sf.corn.converter.json.JsTypeList;
import net.sf.corn.converter.json.JsTypeObject;
import net.sf.corn.converter.json.JsTypeSimple;
import net.sf.corn.converter.json.JsonConverter;
import net.sf.corn.converter.json.JsonStringParser;

import org.springframework.beans.ConversionNotSupportedException;

/**
 * This class represents service requests to JSON-RPC services. JsonRpc client
 * and dispatcher uses this class internally. 
 * The following Java Script class shows a sample request message structure.
 * <code><br>
 *  {<br>
 *   "id":"64f4f4b0-79ee-45f0-9b47-77836bc74ce8",<br>
 *   "service":"service",<br>
*   "operation":"operation",<br>   
 *   "header":{ <br>
 *     "headerField1": "headerVal",<br>
 *     "headerField2": "headerVal",<br>
 *  },<br>
 *   
 *   "parameters":{"param1","param2"}<br> 
 *   }<br>
 * </code><br>
 * <h2>Message Parts</h2> 
 * <h3>id</h3> 
 * This part should contain a unique message
 * identifier for the message correlation. Results of operations will be sent to
 * the clients by using same identifier. This would help clients to correlate
 * their responses from server. 
 * <h3>service</h3> 
 * The remote service name
 * <h3>operation</h3>
 * The remote service operation nam 
 * <h3>header</h3> 
 * The header part of the message can contain anything that you want
 * to send to the server beside parameters.The header part is not accessible by
 * services. Interceptors can access header fields to apply generic policies to
 * service calls. For example you can send additional security credentials to
 * create your own security. 
 * <h3>parameters</h3> 
 * This part contains the target operation's parameters.
 * 
 * 
 * @author Serhat Dirik
 * 
 */
class JsonRpcRequest implements Serializable{

	private static final long serialVersionUID = -312180336124254589L;
	public static final String SERVICE_ATTRIBUTE = "service";
	public static final String OPERATION_ATTRIBUTE = "operation";
	public static final String PARAMETERS_ATTRIBUTE = "params";
	public static final String REQUESTID_ATTRIBUTE = "id";
	public static final String HEADER_ATTRIBUTE = "header";
	private JsTypeComplex enclosedRequest;
	private String serviceName;
	private String operationName;
	private JsTypeList parameters;
	private String requestId;
	private Map<String, Object> header = new HashMap<String, Object>();
	private transient JsonConverter converter = new JsonConverter();
	/**
	 * Creates new empty message
	 */
	public JsonRpcRequest() {
		enclosedRequest = new JsTypeComplex();
	}

	/**
	 * Creates new empty message
	 * @throws ParsingException 
	 * @throws JsonException 
	 * @throws ConversionNotSupportedException 
	 * @throws ConversionException 
	 */
	public JsonRpcRequest(String jsonString) throws ParsingException, ConversionException  {
		JsTypeComplex json_req = (JsTypeComplex) JsonStringParser.parseJsonString(jsonString);
		setEnclosedRequest(json_req);
	}
	
 
	/**
	 * Resets all parts of the message and assigns the from the given message
	 * parts
	 * 
	 * @param jsonReq
	 * @throws ConversionException
	 * @throws ConversionNotSupportedException
	 */
	@SuppressWarnings("unchecked")
	private void setEnclosedRequest(JsTypeComplex jsonReq) throws ConversionException {
		this.enclosedRequest = jsonReq;
		
		JsTypeObject jsObj = null;
		if (enclosedRequest != null) {
			jsObj = jsonReq.get(REQUESTID_ATTRIBUTE);
			requestId = jsObj != null ? jsObj.toStringValue() : null;
			jsObj = jsonReq.get(SERVICE_ATTRIBUTE);
			serviceName = jsObj != null ? jsObj.toStringValue() : null;
			if (serviceName == null) {
				throw new ConversionException("service name not found");
			}
			jsObj = jsonReq.get(OPERATION_ATTRIBUTE);
			operationName = jsObj != null ? jsObj.toStringValue() : null;
			if (operationName == null) {
				throw new ConversionException("operation name not found");
			}
			jsObj = jsonReq.get(PARAMETERS_ATTRIBUTE);
			if(jsObj == null || ! jsObj.isList()){
				parameters = new JsTypeList();
			}else{
				parameters = (JsTypeList) jsonReq.get(PARAMETERS_ATTRIBUTE);
			}
			header = new HashMap<String, Object>(0);
			JsTypeObject hobj = jsonReq.get(HEADER_ATTRIBUTE);
			JsTypeComplex headerJSON =hobj != null && hobj.isComplex() ? (JsTypeComplex) jsonReq.get(HEADER_ATTRIBUTE) : null;
			if(headerJSON != null){
				if(headerJSON.get("map") != null){
					header = (Map<String, Object>) converter.toJava(headerJSON);
				}else{
					for(String key:headerJSON.keys()){
						header.put(key,  converter.toJava(headerJSON.get(key)));
						setHeader(header);
					}
				}
				
				/*for (String key : headerJSON.keys()) {
					// JSONObject headerItem = headerJSON.getJSONObject(key);
					JsTypeObject headerItem = headerJSON.get(key);
					// if(headerItem == null) continue;
					Object headerObject = converter.toJava(headerItem);
					// Object headerObj = transformer.toJava(headerItem);
					header.put(key, headerObject);
				}*/
			}
		} else {
			throw new ConversionException("Empty request in");
		}

	}

	/**
	 * Assigns the given parameter to the service name. The service name and
	 * message name will be merged in method part of the message in dotted form
	 * like "servicename.methodname".
	 * 
	 * @param serviceName
	 * @throws JSONException
	 */
	public void setServiceName(String serviceName) {
		enclosedRequest.remove(SERVICE_ATTRIBUTE);
		try {
			enclosedRequest.putObject(SERVICE_ATTRIBUTE, 
					new JsTypeSimple(JsTypeObject.quote(serviceName),null));
			this.serviceName = serviceName;
		} catch (ConversionException | ConversionNotSupportedException e) {
			e.printStackTrace();
		}
	}
	
 
	/**
	 * Assigns the given parameter to the method name. The method name and
	 * service name will be merged in method part of the message in dotted form
	 * like "servicename.methodname".
	 * 
	 * @param methodName
	 * @throws JSONException
	 */
	public void setOperationName(String opName) {
		enclosedRequest.remove(OPERATION_ATTRIBUTE);
		try {
			enclosedRequest.putObject(OPERATION_ATTRIBUTE, 
					new JsTypeSimple(JsTypeObject.quote(opName),null));
			this.operationName = opName;
		} catch (ConversionException | ConversionNotSupportedException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Sets given object array as parameters
	 * 
	 * @param parameters
	 * @throws ConversionNotSupportedException 
	 * @throws ConversionException 
	 * @throws JSONException
	 */
	public void setParameters(Object[] parameters) throws ConversionException, ConversionNotSupportedException  {
		JsTypeList jsArr = (JsTypeList) converter.toTargetType(parameters);
		setParameters(jsArr);
	}

	/**
	 * Sets given JSONArray as parameters
	 * 
	 * @param parameters
	 * @throws JSONException
	 */
	public void setParameters(JsTypeList parameters){
		this.parameters = parameters;
		enclosedRequest.remove(PARAMETERS_ATTRIBUTE);
		enclosedRequest.put(PARAMETERS_ATTRIBUTE, parameters);
	}

	/**
	 * Sets the unique request id for the message correlation
	 * 
	 * @param requestId
	 * @throws ConversionNotSupportedException 
	 * @throws ConversionException 
	 * @throws JSONException
	 */
	public void setRequestId(String requestId){
		this.requestId = requestId;
		enclosedRequest.remove(REQUESTID_ATTRIBUTE);
		enclosedRequest.put(REQUESTID_ATTRIBUTE,  new JsTypeSimple(JsTypeObject.quote(requestId),null));
	}

	/**
	 * Sets the header part of the message
	 * 
	 * @param header
	 * @throws JSONException
	 * @throws ConversionException
	 * @throws ConversionNotSupportedException
	 */
	public void setHeader(Map<String, Object> header) throws ConversionException, ConversionNotSupportedException{
		this.header = header;
		enclosedRequest.remove(HEADER_ATTRIBUTE);
		if (header == null)
			return;
		JsTypeObject jsHdr = converter.toTargetType(header);
		enclosedRequest.put(HEADER_ATTRIBUTE, jsHdr);
	}

	/**
	 * Gives headers as unmodifiable collection
	 * 
	 * @return unmodifiable header part
	 */
	public Map<String, Object> getHeader() {
		return Collections.unmodifiableMap(header);
	}

	/**
	 * Adds given field to the header
	 * 
	 * @param fieldName
	 * @param value
	 * @throws JSONException
	 * @throws ConversionException
	 * @throws ConversionNotSupportedException
	 */
	public void putToHeader(String fieldName, Object value) throws ConversionException, ConversionNotSupportedException{
		if (header == null)
			header = new HashMap<String, Object>();
		header.put(fieldName, value);
		setHeader(header);
	}

	/**
	 * Removes the requested field form the header
	 * 
	 * @param fieldName
	 * @throws JSONException
	 * @throws ConversionException
	 * @throws ConversionNotSupportedException
	 */
	public void removeFromHeader(String fieldName) throws ConversionException, ConversionNotSupportedException{
		if (header == null)
			return;
		header.remove(fieldName);
		setHeader(header);
	}

	public String getServiceName() {
		return serviceName;
	}

	public String getOperationName() {
		return operationName;
	}

	public JsTypeList getParameters() {
		return parameters;
	}

	public String getRequestId() {
		return requestId;
	}

	/**
	 * Converts message to JSONObject.
	 * 
	 * @return JSON Object
	 */
	public JsTypeObject toJSON() {
		return enclosedRequest;
	}

	/**
	 * Converts message to java script.
	 * 
	 * @return text in javascript notation
	 */
	public String toString() {
		return enclosedRequest == null ? null : enclosedRequest.toString();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((operationName == null) ? 0 : operationName.hashCode());
		result = prime * result
				+ ((parameters == null) ? 0 : parameters.hashCode());
		result = prime * result
				+ ((requestId == null) ? 0 : requestId.hashCode());
		result = prime * result
				+ ((serviceName == null) ? 0 : serviceName.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		JsonRpcRequest other = (JsonRpcRequest) obj;
		if (operationName == null) {
			if (other.operationName != null)
				return false;
		} else if (!operationName.equals(other.operationName))
			return false;
		if (parameters == null) {
			if (other.parameters != null)
				return false;
		} else if (!parameters.equals(other.parameters))
			return false;
		if (requestId == null) {
			if (other.requestId != null)
				return false;
		} else if (!requestId.equals(other.requestId))
			return false;
		if (serviceName == null) {
			if (other.serviceName != null)
				return false;
		} else if (!serviceName.equals(other.serviceName))
			return false;
		return true;
	}

}
